module StarkitBanking
  module YBL
    module Beneficiary
      module IMT
        class Add
          include ApiClient

          private

          def invoke(env, req, callbacks)
            ApiBanking::InstantMoneyTransferService.add_beneficiary(env, request, callbacks)
          end

          def credentials(beneficiary)
            beneficiary.credentials
          end

          def request(beneficiary)

            beneficiaryAddress = ApiBanking::InstantMoneyTransferService::AddBeneficiary::Address.new()
            request = ApiBanking::InstantMoneyTransferService::AddBeneficiary::Request.new()

            @req_reference = request.uniqueRequestNo = beneficiary.id

            request.appID = app_id(beneficiary)
            request.customerID = customer_id(beneficiary)

            request.beneficiaryMobileNo = beneficiary.mobile
            request.beneficiaryName = beneficiary.name
            beneficiaryAddress.addressLine = beneficiary.address_line
            beneficiaryAddress.cityName = beneficiary.city
            beneficiaryAddress.postalCode = beneficiary.postal_code
            request.beneficiaryAddress = beneficiaryAddress

            request
          end

          def customer_id(beneficiary)
            beneficiary.customer_id #TODO customer_id column to be added to bank_beneficiaries table
          end

          def app_id(beneficiary)
            beneficiary.service_subscriptions.find_by('IMT TRANSFER').app_id
          end

        end

        class Delete
          include ApiClient

          private

          def invoke(env, req)
            ApiBanking::InstantMoneyTransferService.delete_beneficiary(env, request)
          end

          def credentials(beneficiary)
            beneficiary.credentials
          end

          def request(beneficiary)

            request = ApiBanking::InstantMoneyTransferService::DeleteBeneficiary::Request.new()

            request.uniqueRequestNo = SecureRandom.uuid.gsub!('-','')
            request.appID = app_id(beneficiary)
            request.customerID = customer_id(beneficiary)
            request.beneficiaryMobileNo = beneficiary.mobile_no

            request
          end

          def customer_id(beneficiary)
            beneficiary.customer_id #TODO customer_id column to be added to bank_beneficiaries table
          end

          def app_id(beneficiary)
            beneficiary.service_subscriptions.find_by('IMT TRANSFER').app_id
          end

        end
      end
    end
  end
end

